require 'mixlib/shellout'

module OnetableTerminator
  module Iptables
    class Exec
      attr_reader :parser, :dry_run

      IPTABLES_BINARY = '/sbin/iptables'.freeze
      OPENNEBULA_CHAIN = 'opennebula'.freeze
      COMMAND_LIST_OPENNEBULA_CHAIN = [IPTABLES_BINARY, '-L', OPENNEBULA_CHAIN, '-n', '--line-numbers'].freeze
      COMMAND_DELETE_OPENNEBULA_RULE = [IPTABLES_BINARY, '-D', OPENNEBULA_CHAIN].freeze
      COMMAND_FLUSH_OPENNEBULA_CHAIN = [IPTABLES_BINARY, '-F'].freeze
      COMMAND_DELETE_OPENNEBULA_CHAIN = [IPTABLES_BINARY, '-X'].freeze

      def initialize(dry_run = false)
        @parser = OnetableTerminator::Iptables::Parser.new
        @dry_run = dry_run
      end

      def load_opennebula_chain
        output = run_command COMMAND_LIST_OPENNEBULA_CHAIN, 'Cannot retrieve rules for opennebula iptables chain', safe: true, log_only: true

        logger.debug('Output:')
        logger.debug(output)

        parser.parse_rules output
      end

      def delete_opennebula_rule(rule_number)
        command = COMMAND_DELETE_OPENNEBULA_RULE.dup
        command << rule_number.to_s

        run_command command, 'Cannot delete opennebula chain rule'
      end

      def flush_opennebula_chain(chain)
        command = COMMAND_FLUSH_OPENNEBULA_CHAIN.dup
        command << chain

        run_command command, 'Cannot flush opennebula chain'
      end

      def delete_opennebula_chain(chain)
        command = COMMAND_DELETE_OPENNEBULA_CHAIN.dup
        command << chain

        run_command command, 'Cannot delete opennebula chain'
      end

      private

      def run_command(command, error_msg, options = {})
        logger.debug("Running command: #{command.inspect}")
        output = ''

        if !dry_run || options[:safe]
          iptables = Mixlib::ShellOut.new(command)
          iptables.run_command

          output = iptables.stdout
          if iptables.error?
            if options[:log_only]
              logger.warn "Command execution error: #{error_msg}: #{iptables.stderr}"
              return ''
            end

            raise OnetableTerminator::Errors::CommandExecutionError, "#{error_msg}: #{iptables.stderr}" if iptables.error?
          end
        end

        output
      end
    end
  end
end
